Remote integration tests
========================

LAMP Stack
----------

In order to run remote integration tests you need to have up and running LAMP stack
as specified in official Skadate software site (http://skadate.com/requirement.php).

Skadate source directory
------------------------

You need to specify path to Skadate software sources directory
defining the `SKADATE_SOURCE_DIR` environment variable, for example:

```
$> export SKADATE_SOURCE_DIR=/home/skadate/release-versions/9.3.3144
```

Temporary deployment directory
------------------------------

Configure your Apache server so that you will be able to
access the `/tmp/skadate/local` and `/tmp/skadate/remote` directories
through `http://skadate-local-test/` and `http://skadate-remote-test/` accordingly.

In order to do this, you need to put the following into your Apache's `httpd-vhosts.conf`:

```
<Directory "/tmp/skadate">
  Options Indexes FollowSymLinks
  AllowOverride All

  Order allow,deny
  Allow from all
</Directory>

<VirtualHost 127.0.0.1:80>
  DocumentRoot "/tmp/skadate/local"
  ServerName   skadate-local-test
</VirtualHost>

<VirtualHost 127.0.0.1:80>
  DocumentRoot "/tmp/skadate/remote"
  ServerName   skadate-remote-test
</VirtualHost>
```

And the following into your `/etc/hosts`:

```
127.0.0.1       skadate-local-test skadate-remote-test
```

Environment variables
---------------------

You can redefine the following environment variables:

* `SKADATE_LOCAL_DIR` default: `"/tmp/skadate/local"`
* `SKADATE_LOCAL_HOST` default: `"skadate-local-test"`
* `SKADATE_REMOTE_DIR` default: `"/tmp/skadate/remote"`
* `SKADATE_REMOTE_HOST` default: `"skadate-remote-test"`

Debugging
---------

Set the `VERBOSE` environment variable to `true` to see what's happening in the background.

Big files downloading stress-test
---------------------------------

When running `lib/remote/download_file_spec.rb` you can also
define your own (probably very big) file to run this tests with.

> Warning: be careful, this may seriously hit your filesystem!

To do this, define `SKADATE_BIG_FILE_PATH` environment variable specifying
the full path to a big file in your filesystem.

For example:

```
$> export SKADATE_BIG_FILE_PATH="$HOME/Movies/Terminator IV - Salvation.mkv"
```

Then, run the test:

```
$> rspec -f d integration/lib/remote/download_file_spec.rb
```

To return the default behaviour, simply `unset` the variable:

```
$> unset SKADATE_BIG_FILE_PATH
```
