require File.expand_path('../spec_helper', __dir__)
require 'skadategems/dev/cli/skadate'
require 'fileutils'

module SkadateGems::Dev

  describe '$> skadate init' do

    def skadate_init!(arguments)
      puts "skadate init #{arguments.join ' '}" if verbose?
      @stdout = capture(:stdout) do
        CLI::Skadate.start( %w[init].concat(arguments) )
      end
    end

    before(:each) do
      FileUtils.mkdir_p(skadate_local_dir, verbose: verbose?) unless File.exists?(skadate_local_dir)
      FileUtils.cd(skadate_local_dir, verbose: verbose?)
    end

    let(:dev_dir_path) { File.join(skadate_local_dir, '.dev') }

    context '(without an arguments)' do
      it 'creates a `.dev/` directory if not exists' do
        expect do
          skadate_init! []
        end.to change { File.exists? dev_dir_path }.to true
      end

      it 'generates a `.dev/.htaccess` file with the "Deny from all" instruction' do
        htaccess_file_path = File.join(dev_dir_path, '.htaccess')

        expect do
          skadate_init! []
        end.to change { File.exists? htaccess_file_path }.to true

        expect(File.read(htaccess_file_path)).to eq <<-HTACCESS
Order Deny,Allow
Deny from all
        HTACCESS
      end
    end

    context '--remote-url REMOTE_URL' do
      it 'generates a `.dev/remote-server.json` configuration file' do
        remote_server_config_path = File.join(dev_dir_path, 'remote-server.json')

        expect do
          skadate_init! ['-r', "http://#{skadate_remote_host}/"]
        end.to change { File.exists? remote_server_config_path }.to true
      end

      it 'generates an `exec.php` (server-side script runner) file' do
        execphp_script_path = File.join(skadate_local_dir, 'exec.php')

        expect do
          skadate_init! ['-r', "http://#{skadate_remote_host}/"]
        end.to change { File.exists? execphp_script_path }.to true
      end

      context 'when the `exec.php` file is uploaded to a remote server' do
        before(:each) do
          skadate_init! ['-r', "http://#{skadate_remote_host}/"]
          FileUtils.mkdir(skadate_remote_dir, verbose: verbose?) unless File.exists?(skadate_remote_dir)
          FileUtils.mv('exec.php', skadate_remote_dir, verbose: verbose?)
        end

        it 'makes `skadate remote ping` invoke successful' do
          puts 'skadate remote ping' if verbose?

          output = capture(:stdout) do
            CLI::Remote.start %w[ping]
          end

          expect(output).to eq <<-STDOUT
=> http://#{skadate_remote_host}/exec.php
#> 200 [OK]
>> pong!
          STDOUT
        end
      end
    end

  end

end
