require File.join(__dir__, 'spec_helper')
require 'skadategems/cli/skadate'

module SkadateGems
  module CLI

    describe Skadate do
      before(:all) { FileUtils.cd(SKADATE_STUB_DIR) }

      describe '$> skadate software about' do
        it 'prints out a Skadate software version information' do
          stdout = capture(:stdout) { Skadate.start %w[software about] }
          expect(stdout).to match /^Skadate Software \d\.\d\.\d{4} \(\w+\) \[\w+\]$/
          expect(stdout).to match /^Release date: \w{3,9} \d{1,2}, 20\d{2}$/
          expect(stdout).to include "Copyright (c) 2004-#{Date.today.year} Skalfa LLC\n"
        end
      end

      describe '$> skadate software version' do
        it 'prints out a Skadate software version number' do
          stdout = capture(:stdout) { Skadate.start %w[software version] }
          expect(stdout).to match /^\d\.\d\.\d{4}$/
        end
      end

      describe '$> skadate software build' do
        it 'prints out a Skadate software build number' do
          stdout = capture(:stdout) { Skadate.start %w[software build] }
          expect(stdout).to match /^\d{4}$/
        end
      end

      describe '$> skadate software release-date' do
        it 'prints out a Skadate version release date' do
          stdout = capture(:stdout) { Skadate.start %w[software release-date] }
          expect(stdout).to match /^\w{3,9} \d{1,2}, 20\d{2}$/
        end
      end

      describe '$> skadate software copyright' do
        it 'prints out Skadate copyrights' do
          stdout = capture(:stdout) { Skadate.start %w[software copyright] }
          expect(stdout).to eq "Copyright (c) 2004-#{Date.today.year} Skalfa LLC\n"
        end
      end

    end

  end
end
