# integration/spec_helper.rb

require File.expand_path('../spec/spec_helper', __dir__)
require 'skadategems/dev/cli/skadate'
require 'fileutils'

RSpec.configure do |config|

  config.before(:each) do
    # cleanup sandbox directories
    [skadate_local_dir, skadate_remote_dir].each do |entry|
      if entry.is_a?(String) && (entry['tmp'] || entry['temp']) && entry['skadate']
        FileUtils.rm_r Dir.glob("#{entry}/{*,.dev,.ht*}"), :verbose => verbose?
      else
        raise <<-ERRORMSG
Error: bad tmp dir entry "#{path}".
Explanation: by security reason tmp directories must contain 'tmp' and 'skadate' keywords in their pathname.
        ERRORMSG
      end
    end
  end

  def env_var!(name)
    ENV[name] or raise "`#{name}` is not defined"
  end

  def verbose?
    (verbose = ENV['VERBOSE']) && verbose != '' && verbose.upcase != 'FALSE'
  end

  def skadate_tmp_dir
    @skadate_tmp_dir ||= File.join(File.symlink?('/tmp') ? "/#{File.readlink('/tmp')}" : '/tmp', 'skadate')
  end

  def skadate_source_dir
    @skadate_source_dir ||= env_var! 'SKADATE_SOURCE_DIR'
  end

  def skadate_local_dir
    @skadate_local_dir ||= ENV['SKADATE_LOCAL_DIR'] || File.join(skadate_tmp_dir, 'local')
  end

  def skadate_remote_dir
    @skadate_remote_dir ||= ENV['SKADATE_REMOTE_DIR'] || File.join(skadate_tmp_dir, 'remote')
  end

  def skadate_local_host
    ENV['SKADATE_LOCAL_HOST'] || 'skadate-local-test'
  end

  def skadate_remote_host
    ENV['SKADATE_REMOTE_HOST'] || 'skadate-remote-test'
  end

  def remote_install(options = {})
    FileUtils.cd(skadate_remote_dir, :verbose => verbose?) do
      FileUtils.mkdir('internals', :verbose => verbose?)

      puts '[...] > internals/config.php' if verbose?
      IO.write('internals/config.php', <<-CONFIGPHP)
<?php
define('DIR_SITE_ROOT', '#{skadate_remote_dir}/');
define('SITE_URL', 'http://#{skadate_remote_host}/');
      CONFIGPHP
    end

    FileUtils.cd(skadate_local_dir, :verbose => verbose?) do
      puts "skadate init -r http://#{skadate_remote_host}/" if verbose?
      SkadateGems::Dev::CLI::Skadate.start ['init', '-r', "http://#{skadate_remote_host}/"]
      FileUtils.cp('exec.php', skadate_remote_dir, :verbose => verbose?)
    end
  end

  def local_install(options = {})
  end

  def source_copy(source_file_paths, destination_root = :local, options = {})
    destination_root = skadate_local_dir if destination_root == :local
    destination_root = skadate_remote_dir if destination_root == :remote

    [*source_file_paths].each do |entry|
      src = File.join(skadate_source_dir, entry)
      dest = File.join(destination_root, entry)
      FileUtils.mkdir_p File.dirname(dest), :verbose => verbose?
      FileUtils.cp(src, dest, options.merge(:verbose => verbose?))
    end
  end

  # @param bytes [Fixnum] value in bytes
  # @return [String] human readable format
  def human_readable_bytes(bytes)
    units = %W(B KiB MiB GiB TiB)

    size, unit = units.reduce(bytes.to_f) do |(fsize, _), type|
      fsize > 512 ? [fsize / 1024, type] : (break [fsize, type])
    end

    "#{size > 9 || size.modulo(1) < 0.1 ? '%d' : '%.1f'} %s" % [size, unit]
  end

end
