require 'execphp'
require 'skadategems/dev/remote/directory'
require 'skadategems/dev/remote/configs'

module SkadateGems
  module Dev

    # Represents remote Skadate-driven application.
    class Remote

      attr_reader :accessor

      def initialize(config_filename)
        @accessor = ExecPHP::RemoteServer.from_file(config_filename)
      end

      def exec(options = {}, &block)
        batch = ScriptBatch.new

        if options[:include]
          [*options[:include]].each do |include_name|
            batch.include_file ::File.join(__dir__, "remote/includes/#{include_name}.php")
          end
        end

        block.call(batch)

        @accessor.exec(batch)
      end

      # @return [Remote::Directory] remote source directory accessor
      def directory(path)
        Directory.new(self, path, 0)
      end

      # @return [Remote::Directory] remote source root directory
      def root
        @root ||= directory '/'
      end

      # @return [Remote::Configs] remote application configs accessor
      def configs
        @config ||= Configs.new(self)
      end

      def create_mysql_dump(&block)
        response = exec do |script|
          script << <<-PHPSCRIPT
require_once DIR_INTERNALS . 'system.const.php';

if (strpos(DIR_INTERNAL_C, DIR_SITE_ROOT) !== 0) {
  header('HTTP/1.1 500 Internal Server Error');
  exit('Error: unable to detect path to `internal_c/`.');
}

function program_available($bin) {
  $output = null;
  $result = -1;
  exec("type $bin>/dev/null 2>&1", $output, $result);
  return $result === 0;
}

if (!program_available('mysqldump')) {
  header('HTTP/1.1 500 Internal Server Error');
  exit('Error: `mysqldump` is not available');
}

$tmp_dir = DIR_INTERNAL_C;
$tmp_dir_path = substr($tmp_dir, strlen(DIR_SITE_ROOT));

$uniqid = uniqid();
$sql_filename = "$uniqid.sql";

$db_host = DB_HOST;
$db_user = DB_USER;
$db_pass = DB_PASS;
$db_name = DB_NAME;

$output = null;
$result = -1;
exec("cd '$tmp_dir' && mysqldump --host='$db_host' --user='$db_user' --password='$db_pass' '$db_name' > '$sql_filename'", $output, $result);
if (!file_exists($tmp_dir . $sql_filename)) {
  header('HTTP/1.1 500 Internal Server Error');
  exit('`mysqldump` error: ' . join($output, "\n"));
}

if (program_available('tar')) {
  $tar_filename = "$uniqid.tar.gz";

  $output = null;
  $result = -1;
  exec("cd '$tmp_dir' && tar -czf '$tar_filename' '$sql_filename'", $output, $result);
  if (file_exists($tmp_dir . $tar_filename)) {
    unlink($tmp_dir . $sql_filename);
    exit('["' . $tmp_dir_path . $tar_filename . '", ' . filesize($tmp_dir . $tar_filename) . ']');
  }
}

exit('["' . $tmp_dir_path . $sql_filename . '", ' . filesize($tmp_dir . $sql_filename) . ']');
          PHPSCRIPT
        end

        raise "#{response.code} [#{response.message}]:\n >> #{response.body}" if response.code != '200'

        path, size = JSON.parse(response.body)
        block.call File.new(self, path, size)

        response = exec do |script|
          script << <<-PHPSCRIPT
echo unlink(DIR_SITE_ROOT . '#{path}') ? '1' : '0';
          PHPSCRIPT
        end

        raise "#{response.code} [#{response.message}]:\n >> #{response.body}" if response.code != '200'

        response.body == '1'
      end

      # Detect the name of the `userfiles` directory requesting a remote application.
      # @return [Remote::Directory] remote `userfiles` directory
      def userfiles
        response = exec do |script|
          script << <<-PHPSCRIPT
require_once DIR_INTERNALS . 'system.const.php';

if (strpos(DIR_USERFILES, DIR_SITE_ROOT) !== 0) {
  header('HTTP/1.1 500 Internal Server Error');
  exit('Error: unable to detect path to `/userfiles`.');
}

echo substr(DIR_USERFILES, strlen(DIR_SITE_ROOT) - 1, -1);
          PHPSCRIPT
        end

        raise "#{response.code} [#{response.message}]:\n >> #{response.body}" if response.code != '200'

        directory(response.body)
      end

      # PHP script batch.
      class ScriptBatch < ExecPHP::ScriptBatch

        # Initializes a remote script batch which automatically
        # finds and requires Skadate's `config.php` file.
        def initialize
          super do |script|
            script << <<-PHPSCRIPT
$try_dir = dirname(__FILE__);
while (!(file_exists($config_file = $try_dir . '/internals/config.php')
	|| file_exists($config_file = $try_dir . '/internals/$config.php')))
{
	$prev_dir = $try_dir;
	if ($prev_dir == ($try_dir = dirname($try_dir))) {
		throw new Exception('Error: unable to find a config file');
	}
}

require_once $config_file;

define('DIR_INTERNALS', dirname($config_file) . DIRECTORY_SEPARATOR);
            PHPSCRIPT
          end
        end

      end

    end

  end
end
