require 'skadategems/dev/remote/directory'
require 'skadategems/dev/remote/file'
require 'thor/shell'

module SkadateGems::Dev
  class Remote

    class CloneLogger < Thor::Base.shell

      def self.start(log_file_path, &block)
        ::File.open(log_file_path, 'w') do |file_io|
          block.call new(file_io)
        end
      end

      def initialize(file_io)
        @file_io = file_io
        super()
      end

      # @param entry [Remote::Entry] remote fs entry
      # @param status [Symbol] log entry status
      # @param label [String] log entry badge label
      # @yield if block is given expecting that it returns an entry processing status
      def entry(entry, status = nil, label = nil)
        icon = if status == :skipped
          '*'
        elsif entry.is_a?(Directory)
          '='
        elsif entry.is_a?(File)
          '~'
        end

        prefix = "#{'  ' * padding}#{icon}>"

        print(if entry.is_a?(Directory)
                "#{prefix} #{set_color(entry.path, :blue)}"
              elsif entry.is_a?(File)
                "#{prefix} #{set_color(entry.basename, :white)} (#{set_color(entry.friendly_size, :cyan)})"
              else
                "#{prefix} #{set_color(entry.to_s, :on_black)}"
              end)

        write(if entry.is_a?(Directory)
                "#{prefix} #{entry.path}"
              elsif entry.is_a?(File)
                "#{prefix} #{entry.basename} (#{entry.friendly_size})"
              else
                "#{prefix} #{entry}"
              end)

        if status == :skipped
          label ||= 'skipped'
          print(" [#{set_color(label, :magenta)}]")
          write(" [#{label}]")
        end

        if block_given?
          print ' ...'
          stdout.flush

          status = yield

          label, color =
            case status
              when :downloaded
                ['downloaded', :green]
              when :unchanged
                ['unchanged', :cyan]
              else
                # no status badge
            end

          if label
            print "\b" * 4 if can_display_colors?
            print(" [#{set_color(label, color)}]")
            write(" [#{label}]")
          end
        end

      rescue
        print "\b" * 4 if can_display_colors?
        print " [#{set_color('error', :red)}]"
        write " [error]\n#{'>' * 30}\n[#{$!.class.name}]: #{$!.message}\n#{'-' * 30}"
        raise $!
      ensure
        print "\n"
        write "\n"
        stdout.flush
      end

      # @param line [String] string line to log
      # @param color [Symbol] string color to colorize if supported
      def puts(line, color = nil)
        unless mute?
          super set_color(line, color)
          stdout.flush
        end

        write "#{line}\n"
      end

      def set_color(string, *colors)
        can_display_colors? ? super(string, *colors) : string
      end

      def print(obj, *rest)
        super unless mute?
      end

      def write(string)
        @file_io << string if @file_io
      end

    end

  end
end
