require 'json'

module SkadateGems
  module Dev

    class Remote

      # Represents remote application configurations accessor.
      class Configs

        # @param remote [Remote] skadate remote app
        def initialize(remote)
          @remote = remote
        end

        # Configuration value accessor method.
        # @param section_path_or_id [Fixnum|String] config section path or `config_id`
        def [](section_path_or_id)
          case section_path_or_id.class.to_s
            when 'String'
              raise 'Getting remote configs by path is currently not implemented'
            when 'Fixnum'
              get_by_id(section_path_or_id)
            else
              raise ArgumentError, "unexpected `section_path_or_id` type #{section_path_or_id.class}"
          end
        end

        # @param config_id [Fixnum] configuration id
        def get_by_id(config_id)
          response = @remote.exec include: 'mysql_connect' do |script|
            script << <<-PHPSCRIPT
$result = $mysql->query("SELECT `name`, `value` FROM `".TBL_CONFIG."` WHERE `config_id`=#{config_id}");
$config = $result->fetch_object();
echo '{ ';
echo '"name": "'.$config->name.'", ';
echo '"value": '.$config->value;
echo ' }';
            PHPSCRIPT
          end

          result = JSON.parse(response.body)

          Config.new(@remote, config_id, result['name'], result['value'])
        end

        # Represents remote application config entry.
        class Config

          # @return [Fixnum] config entry id
          attr_reader :id

          # @return [String] config entry name
          attr_reader :name

          # @return [mixed] config entry value
          attr_reader :value

          # @return [nil | :updated | :not_modified] config value update status
          attr_reader :update_status

          # @param remote [Remote] remote skadate app
          # @param id [Fixnum] config id
          # @param name [String] config name
          # @param value [mixed] config value
          def initialize(remote, id, name, value)
            @remote, @id, @name, @value = remote, id, name, value
          end

          # Perform remote application config value update.
          # Changes the #update_status value.
          # @param value [#to_json] json serializable value
          def value=(value)
            if value == @value
              @update_status = :not_modified
              return
            end

            response = @remote.exec(include: 'mysql_connect') do |script|
              script << <<-PHPSCRIPT
$value = $mysql->real_escape_string('#{value.to_json.gsub(/['\\]/, '\\\\\0')}');
$mysql->query("UPDATE `".TBL_CONFIG."` SET `value`='$value' WHERE `config_id`=#{@id}");
echo $mysql->affected_rows;
              PHPSCRIPT
            end

            @update_status = (response.body == '1') ? :updated : :not_modified
          end

        end

      end

    end

  end
end
