require 'json'
require 'skadategems/dev/remote/file'

module SkadateGems
  module Dev

    class Remote

      # Represents a remote directory.
      class Directory < File

        # @attr list_hidden [boolean]
        attr_accessor :list_hidden

        # @return [boolean] always true
        def directory?
          true
        end

        # @return [String] always '/'
        def extname
          '/'
        end

        # List remote directory files.
        # @return [Array < Directory | File >] where each entry
        #   may be either a `Remote::Directory` or a `Remote::File` accessor object.
        def list
          path = self.path.dup
          path[0] = '' if path[0] == '/'
          path[-1] = '' if path[-1] == '/'

          response = @remote.exec do |batch|
            batch << <<-PHPSCRIPT
$dir = dir(DIR_SITE_ROOT . '#{path.gsub(/['\\]/, '\\\\\0')}');

$list = array();
while (false !== ($file = $dir->read())) {
  if (#{if list_hidden
          '$file === \'.\' || $file === \'..\''
        else
          '$file{0} === \'.\' && $file !== \'.htaccess\''
        end}) {
    continue;
  }

  $path = "$dir->path/$file";

  if ($path === $_SERVER['SCRIPT_FILENAME']) {
    continue;
  }

  if (is_file($path)) {
    $type = 'f';
  } elseif (is_dir($path)) {
    $type = 'd';
  } else {
    continue;
  }

  $list[] = array($file, $type, filesize($path));
}

echo json_encode($list);
            PHPSCRIPT
          end

          result = []

          JSON.parse(response.body).each do |entry|
            name, type, size = entry
            _path = ::File.join(self.path, name)

            result << if type == 'd'
              Directory.new(@remote, _path, size)
            else
              File.new(@remote, _path, size)
            end
          end

          result.sort! { |a, b| a.basename <=> b.basename }
        end
      end

    end

  end
end
